@@ -188,9 +188,13 @@ help (void)
 static int
 download (const struct optstruct *opts, const char *cfgfile)
 {
+    time_t currtime;
     int ret = 0, try = 1, maxattempts = 0;
     const struct optstruct *opt;
 
+    time(&currtime);
+    logg("ClamAV update process started at %s", ctime(&currtime));
+    logg("*Using IPv6 aware code\n");
 
     maxattempts = optget (opts, "MaxAttempts")->numarg;
     logg ("*Max retries == %d\n", maxattempts);
@@ -227,6 +231,7 @@ download (const struct optstruct *opts, const char *cfgfile)
                     {
                         logg ("Update failed. Your network may be down or none of the mirrors listed in %s is working. Check https://www.clamav.net/documents/official-mirror-faq for possible reasons.\n", cfgfile);
                     }
+                    try = 1;
                 }
 
             }
@@ -301,7 +306,7 @@ main (int argc, char **argv)
     {
         help ();
         optfree (opts);
-        return 0;
+        return FC_SUCCESS;
     }
 
     /* check foreground option from command line to override config file */
@@ -339,7 +344,7 @@ main (int argc, char **argv)
     {
         print_version (optget (opts, "DatabaseDirectory")->strarg);
         optfree (opts);
-        return 0;
+        return FC_SUCCESS;
     }
 
     if (optget (opts, "HTTPProxyPassword")->enabled)
@@ -494,7 +499,7 @@ main (int argc, char **argv)
 
     if (optget (opts, "list-mirrors")->enabled)
     {
-        if (mirman_read ("mirrors.dat", &mdat, 1) == -1)
+        if (mirman_read("mirrors.dat", &mdat, 1) != FC_SUCCESS)
         {
             printf ("Can't read mirrors.dat\n");
             optfree (opts);
@@ -503,7 +508,7 @@ main (int argc, char **argv)
         mirman_list (&mdat);
         mirman_free (&mdat);
         optfree (opts);
-        return 0;
+        return FC_SUCCESS;
     }
 
     if ((opt = optget (opts, "PrivateMirror"))->enabled)
@@ -725,7 +730,7 @@ main (int argc, char **argv)
 
     cl_cleanup_crypto();
 
-    return (ret);
+    return ret > 1 ? ret : 0;
 }
 
 int is_valid_hostid(void)
