@@ -128,6 +128,7 @@ static const struct ftmap_s {
     { "CL_TYPE_OOXML_HWP",	CL_TYPE_OOXML_HWP	},
     { "CL_TYPE_PS",		CL_TYPE_PS		},
     { "CL_TYPE_MHTML",		CL_TYPE_MHTML		},
+    { "CL_TYPE_LNK", 		CL_TYPE_LNK		},
     { NULL,			CL_TYPE_IGNORED		}
 };
 
@@ -215,7 +216,7 @@ cli_file_t cli_filetype(const unsigned char *buf, size_t buflen, const struct cl
 
 int is_tar(const unsigned char *buf, unsigned int nbytes);
 
-/* organize by length, cannot exceed SIZEOF_LH */
+/* organize by length, cannot exceed SIZEOF_LOCAL_HEADER */
 const struct ooxml_ftcodes {
     const char *entry;
     size_t len;
@@ -318,7 +319,7 @@ cli_file_t cli_filetype2(fmap_t *map, const struct cl_engine *engine, cli_file_t
 		    cli_dbgmsg("Recognized POSIX tar file\n");
 		    return CL_TYPE_POSIX_TAR;
 	    }
-	} else if (ret == CL_TYPE_ZIP && bread > 2*(SIZEOF_LH+5)) {
+	} else if (ret == CL_TYPE_ZIP && bread > 2*(SIZEOF_LOCAL_HEADER+5)) {
             const char lhdr_magic[4] = {0x50,0x4b,0x03,0x04};
             const unsigned char *zbuff = buff;
             uint32_t zread = bread;
@@ -328,11 +329,11 @@ cli_file_t cli_filetype2(fmap_t *map, const struct cl_engine *engine, cli_file_t
             int lhc = 0;
             int zi, i, likely_ooxml = 0;
             cli_file_t ret2;
-            
+
             for (zi=0; zi<32; zi++) {
                 znamep = (const unsigned char *)cli_memstr((const char *)znamep, zlen, lhdr_magic, 4);
                 if (NULL != znamep) {
-                    znamep += SIZEOF_LH;
+                    znamep += SIZEOF_LOCAL_HEADER;
                     zlen = zread - (znamep - zbuff);
                     if (zlen > OOXML_DETECT_MAXLEN) {
                         for (i = 0; ooxml_detect[i].entry; i++) {
@@ -366,8 +367,8 @@ cli_file_t cli_filetype2(fmap_t *map, const struct cl_engine *engine, cli_file_t
                 }
 
                 if (znamep == NULL) {
-                    if (map->len-zoff > SIZEOF_LH) {
-                        zoff -= SIZEOF_LH+OOXML_DETECT_MAXLEN+1; /* remap for SIZEOF_LH+filelen for header overlap map boundary */ 
+                    if (map->len-zoff > SIZEOF_LOCAL_HEADER) {
+                        zoff -= SIZEOF_LOCAL_HEADER+OOXML_DETECT_MAXLEN+1; /* remap for SIZEOF_LOCAL_HEADER+filelen for header overlap map boundary */
                         zread = MIN(MAGIC_BUFFER_SIZE, map->len-zoff);
                         zbuff = fmap_need_off_once(map, zoff, zread);
                         if (zbuff == NULL) {
@@ -439,7 +440,7 @@ cli_file_t cli_filetype2(fmap_t *map, const struct cl_engine *engine, cli_file_t
 		    if((encoding = encoding_detect_bom(buff, bread))) {
 			    unsigned char decodedbuff[(MAGIC_BUFFER_SIZE+1)*2];
 			    m_area_t in_area, out_area;
-			    
+
 			    memset(decodedbuff, 0, sizeof(decodedbuff));
 
 			    in_area.buffer = (unsigned char *) buff;
@@ -449,7 +450,7 @@ cli_file_t cli_filetype2(fmap_t *map, const struct cl_engine *engine, cli_file_t
 			    out_area.length = sizeof(decodedbuff);
 			    out_area.offset = 0;
 
-			    /* in htmlnorm we simply skip over \0 chars, allowing HTML parsing in any unicode 
+			    /* in htmlnorm we simply skip over \0 chars, allowing HTML parsing in any unicode
 			     * (multibyte characters will not be exactly handled, but that is not a problem).
 			     * However when detecting whether a file is HTML or not, we need exact conversion.
 			     * (just eliminating zeros and matching would introduce false positives */
