@@ -28,7 +28,6 @@
 #include "filetypes.h"
 #include "others.h"
 #include "execs.h"
-#include "cltypes.h"
 
 struct cli_target_info {
     off_t fsize;
@@ -40,6 +39,7 @@ struct cli_target_info {
 #include "matcher-bm.h"
 #include "matcher-hash.h"
 #include "matcher-pcre.h"
+#include "matcher-byte-comp.h"
 #include "regex_pcre.h"
 #include "fmap.h"
 #include "mpool.h"
@@ -133,6 +133,10 @@ struct cli_matcher {
     uint32_t pcre_reloff_num, pcre_absoff_num;
 #endif
 
+    /* Byte Compare */
+    uint32_t bcomp_metas;
+    struct cli_bcomp_meta **bcomp_metatable;
+
     /* Bytecode Tracker */
     uint32_t linked_bcs;
 
