@@ -1,6 +1,6 @@
 /*
- *  Copyright (C) 2015, 2017 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
- *  Copyright (C) 2007-2010 Sourcefire, Inc.
+ *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, Trog
  *
@@ -68,7 +68,6 @@
 
 #include "clamav.h"
 #include "others.h"
-#include "cltypes.h"
 #include "regex/regex.h"
 #include "ltdl.h"
 #include "matcher-ac.h"
@@ -81,10 +80,12 @@
 #include "readdb.h"
 #include "stats.h"
 
-int (*cli_unrar_open)(int fd, const char *dirname, unrar_state_t *state);
-int (*cli_unrar_extract_next_prepare)(unrar_state_t *state, const char *dirname);
-int (*cli_unrar_extract_next)(unrar_state_t *state, const char *dirname);
-void (*cli_unrar_close)(unrar_state_t *state);
+cl_unrar_error_t (*cli_unrar_open)(const char *filename, void **hArchive, char **comment, uint32_t *comment_size, uint8_t debug_flag);
+cl_unrar_error_t (*cli_unrar_peek_file_header)(void *hArchive, unrar_metadata_t *file_metadata);
+cl_unrar_error_t (*cli_unrar_extract_file)(void* hArchive, const char* destPath, char *outputBuffer);
+cl_unrar_error_t (*cli_unrar_skip_file)(void *hArchive);
+void (*cli_unrar_close)(void *hArchive);
+
 int have_rar = 0;
 static int is_rar_inited = 0;
 
@@ -176,14 +177,17 @@ static void cli_rarload(void) {
     if(is_rar_inited) return;
     is_rar_inited = 1;
 
+    if(have_rar) return;
+
     rhandle = lt_dlfind("libclamunrar_iface", "unrar");
     if (!rhandle)
 	return;
 
-    if (!(cli_unrar_open = (int(*)(int, const char *, unrar_state_t *))lt_dlsym(rhandle, "libclamunrar_iface_LTX_unrar_open")) ||
-	!(cli_unrar_extract_next_prepare = (int(*)(unrar_state_t *, const char *))lt_dlsym(rhandle, "libclamunrar_iface_LTX_unrar_extract_next_prepare")) ||
-	!(cli_unrar_extract_next = (int(*)(unrar_state_t *, const char *))lt_dlsym(rhandle, "libclamunrar_iface_LTX_unrar_extract_next")) ||
-	!(cli_unrar_close = (void(*)(unrar_state_t *))lt_dlsym(rhandle, "libclamunrar_iface_LTX_unrar_close"))
+    if (!(cli_unrar_open = (cl_unrar_error_t(*)(const char *, void **, char **, uint32_t *, uint8_t))lt_dlsym(rhandle, "libclamunrar_iface_LTX_unrar_open")) ||
+		!(cli_unrar_peek_file_header = (cl_unrar_error_t(*)(void *, unrar_metadata_t *))lt_dlsym(rhandle, "libclamunrar_iface_LTX_unrar_peek_file_header")) ||
+		!(cli_unrar_extract_file = (cl_unrar_error_t(*)(void*, const char*, char*))lt_dlsym(rhandle, "libclamunrar_iface_LTX_unrar_extract_file")) ||
+		!(cli_unrar_skip_file = (cl_unrar_error_t(*)(void *))lt_dlsym(rhandle, "libclamunrar_iface_LTX_unrar_skip_file")) ||
+		!(cli_unrar_close = (void(*)(void *))lt_dlsym(rhandle, "libclamunrar_iface_LTX_unrar_close"))
 	) {
 	/* ideally we should never land here, we'd better warn so */
         cli_warnmsg("Cannot resolve: %s (version mismatch?) - unrar support unavailable\n", lt_dlerror());
@@ -258,7 +262,7 @@ const char *cl_strerror(int clerror)
 	case CL_EMEM:
 	    return "Can't allocate memory";
 	case CL_ETIMEOUT:
-	    return "Time limit reached";
+	    return "CL_ETIMEOUT: Time limit reached";
 	/* internal (needed for debug messages) */
 	case CL_EMAXREC:
 	    return "CL_EMAXREC";
@@ -293,14 +297,6 @@ int cl_init(unsigned int initoptions)
 
     cl_initialize_crypto();
 
-    {
-	unrar_main_header_t x;
-	if (((char*)&x.flags - (char*)&x) != 3) {
-	    cli_errmsg("Structure packing not working, got %u offset, expected %u\n",
-		       (unsigned)((char*)&x.flags - (char*)&x), 3);
-	    return CL_EARG;
-	}
-    }
     /* put dlopen() stuff here, etc. */
     if (lt_init() == 0) {
 	cli_rarload();
@@ -328,6 +324,7 @@ struct cl_engine *cl_engine_new(void)
     }
 
     /* Setup default limits */
+    new->maxscantime = CLI_DEFAULT_MAXSCANTIME;
     new->maxscansize = CLI_DEFAULT_MAXSCANSIZE;
     new->maxfilesize = CLI_DEFAULT_MAXFILESIZE;
     new->maxreclevel = CLI_DEFAULT_MAXRECLEVEL;
@@ -620,9 +617,9 @@ int cl_engine_set_num(struct cl_engine *engine, enum cl_engine_field field, long
 	case CL_ENGINE_MAX_RECHWP3:
 	    engine->maxrechwp3 = (uint32_t)num;
 	    break;
-	case CL_ENGINE_TIME_LIMIT:
-	    engine->time_limit = (uint32_t)num;
-	    break;
+    case CL_ENGINE_MAX_SCANTIME:
+        engine->maxscantime = (uint32_t)num;
+        break;
 	case CL_ENGINE_PCRE_MATCH_LIMIT:
 	    engine->pcre_match_limit = (uint64_t)num;
 	    break;
@@ -721,8 +718,8 @@ long long cl_engine_get_num(const struct cl_engine *engine, enum cl_engine_field
 	    return engine->maxiconspe;
     case CL_ENGINE_MAX_RECHWP3:
 	    return engine->maxrechwp3;
-	case CL_ENGINE_TIME_LIMIT:
-            return engine->time_limit;
+    case CL_ENGINE_MAX_SCANTIME:
+        return engine->maxscantime;
 	case CL_ENGINE_PCRE_MATCH_LIMIT:
 	    return engine->pcre_match_limit;
 	case CL_ENGINE_PCRE_RECMATCH_LIMIT:
@@ -802,6 +799,7 @@ struct cl_settings *cl_engine_settings_copy(const struct cl_engine *engine)
     settings->ac_maxdepth = engine->ac_maxdepth;
     settings->tmpdir = engine->tmpdir ? strdup(engine->tmpdir) : NULL;
     settings->keeptmp = engine->keeptmp;
+    settings->maxscantime = engine->maxscantime;
     settings->maxscansize = engine->maxscansize;
     settings->maxfilesize = engine->maxfilesize;
     settings->maxreclevel = engine->maxreclevel;
@@ -856,6 +854,7 @@ int cl_engine_settings_apply(struct cl_engine *engine, const struct cl_settings
     engine->ac_mindepth = settings->ac_mindepth;
     engine->ac_maxdepth = settings->ac_maxdepth;
     engine->keeptmp = settings->keeptmp;
+    engine->maxscantime = settings->maxscantime;
     engine->maxscansize = settings->maxscansize;
     engine->maxfilesize = settings->maxfilesize;
     engine->maxreclevel = settings->maxreclevel;
@@ -936,16 +935,17 @@ int cl_engine_settings_free(struct cl_settings *settings)
 
 void cli_check_blockmax(cli_ctx *ctx, int rc)
 {
-    if (BLOCKMAX && !ctx->limit_exceeded) {
-        cli_append_virus (ctx, "Heuristic.Limits.Exceeded");
+    if (SCAN_HEURISTIC_EXCEEDS_MAX && !ctx->limit_exceeded) {
+        cli_append_virus (ctx, "Heuristics.Limits.Exceeded");
         ctx->limit_exceeded = 1;
         cli_dbgmsg ("Limit %s Exceeded: scanning may be incomplete and additional analysis needed for this file.\n",
             cl_strerror(rc));
     }
 }
 
-int cli_checklimits(const char *who, cli_ctx *ctx, unsigned long need1, unsigned long need2, unsigned long need3) {
-    int ret = CL_SUCCESS;
+cl_error_t cli_checklimits(const char *who, cli_ctx *ctx, unsigned long need1, unsigned long need2, unsigned long need3)
+{
+    cl_error_t ret = CL_SUCCESS;
     unsigned long needed;
 
     /* if called without limits, go on, unpack, scan */
@@ -954,6 +954,9 @@ int cli_checklimits(const char *who, cli_ctx *ctx, unsigned long need1, unsigned
     needed = (need1>need2)?need1:need2;
     needed = (needed>need3)?needed:need3;
 
+    /* Enforce timelimit */
+    ret = cli_checktimelimit(ctx);
+
     /* if we have global scan limits */
     if(needed && ctx->engine->maxscansize) {
         /* if the remaining scansize is too small... */
@@ -982,8 +985,9 @@ int cli_checklimits(const char *who, cli_ctx *ctx, unsigned long need1, unsigned
     return ret;
 }
 
-int cli_updatelimits(cli_ctx *ctx, unsigned long needed) {
-    int ret=cli_checklimits("cli_updatelimits", ctx, needed, 0, 0);
+cl_error_t cli_updatelimits(cli_ctx *ctx, unsigned long needed)
+{
+    cl_error_t ret = cli_checklimits("cli_updatelimits", ctx, needed, 0, 0);
 
     if (ret != CL_CLEAN) return ret;
     ctx->scannedfiles++;
@@ -993,18 +997,33 @@ int cli_updatelimits(cli_ctx *ctx, unsigned long needed) {
     return CL_CLEAN;
 }
 
-int cli_checktimelimit(cli_ctx *ctx)
+/**
+ * @brief Check if we've exceeded the time limit.
+ * If ctx is NULL, there can be no timelimit so just return success.
+ *
+ * @param ctx         The scanning context.
+ * @return cl_error_t CL_SUCCESS if has not exceeded, CL_ETIMEOUT if has exceeded.
+ */
+cl_error_t cli_checktimelimit(cli_ctx *ctx)
 {
+    cl_error_t ret = CL_SUCCESS;
+
+    if (NULL == ctx) {
+        goto done;
+    }
+
     if (ctx->time_limit.tv_sec != 0) {
         struct timeval now;
         if (gettimeofday(&now, NULL) == 0) {
-            if (now.tv_sec < ctx->time_limit.tv_sec)
-                return CL_SUCCESS;
-            if (now.tv_sec > ctx->time_limit.tv_sec || now.tv_usec > ctx->time_limit.tv_usec)
-                return CL_ETIMEOUT;
+            if (now.tv_sec > ctx->time_limit.tv_sec)
+                ret = CL_ETIMEOUT;
+            else if (now.tv_sec == ctx->time_limit.tv_sec && now.tv_usec > ctx->time_limit.tv_usec)
+                ret = CL_ETIMEOUT;
         }
     }
-    return CL_SUCCESS;
+
+done:
+    return ret;
 }
 
 /*
@@ -1078,14 +1097,29 @@ char *cli_hashfile(const char *filename, int type)
 /* Function: unlink
         unlink() with error checking
 */
-int cli_unlink(const char *pathname)
+int cli_unlink(const char* pathname)
 {
-	if (unlink(pathname)==-1) {
-	    char err[128];
-	    cli_warnmsg("cli_unlink: failure - %s\n", cli_strerror(errno, err, sizeof(err)));
-	    return 1;
-	}
-	return 0;
+    if (unlink(pathname) == -1) {
+#ifdef _WIN32
+        /* Windows may fail to unlink a file if it is marked read-only,
+		 * even if the user has permissions to delete the file. */
+        if (-1 == _chmod(pathname, _S_IWRITE)) {
+            char err[128];
+            cli_warnmsg("cli_unlink: _chmod failure - %s\n", cli_strerror(errno, err, sizeof(err)));
+            return 1;
+        } else if (unlink(pathname) == -1) {
+            char err[128];
+            cli_warnmsg("cli_unlink: unlink failure - %s\n", cli_strerror(errno, err, sizeof(err)));
+            return 1;
+        }
+        return 0;
+#else
+        char err[128];
+        cli_warnmsg("cli_unlink: unlink failure - %s\n", cli_strerror(errno, err, sizeof(err)));
+        return 1;
+#endif
+    }
+    return 0;
 }
 
 void cli_virus_found_cb(cli_ctx * ctx)
@@ -1094,11 +1128,11 @@ void cli_virus_found_cb(cli_ctx * ctx)
         ctx->engine->cb_virus_found(fmap_fd(*ctx->fmap), (const char *)*ctx->virname, ctx->cb_ctx);
 }
 
-int cli_append_possibly_unwanted(cli_ctx * ctx, const char * virname)
+cl_error_t cli_append_possibly_unwanted(cli_ctx *ctx, const char *virname)
 {
-    if (SCAN_ALL)
+    if (SCAN_ALLMATCHES)
         return cli_append_virus(ctx, virname);
-    else if (ctx->options & CL_SCAN_HEURISTIC_PRECEDENCE)
+    else if (SCAN_HEURISTIC_PRECEDENCE)
         return cli_append_virus(ctx, virname);
     else if (ctx->num_viruses == 0 && ctx->virname != NULL && *ctx->virname == NULL) {
         ctx->found_possibly_unwanted = 1;
@@ -1114,16 +1148,16 @@ int cli_append_virus(cli_ctx * ctx, const char * virname)
         return CL_CLEAN;
     if (ctx->fmap != NULL && (*ctx->fmap) != NULL && CL_VIRUS != cli_checkfp_virus((*ctx->fmap)->maphash, (*ctx->fmap)->len, ctx, virname))
         return CL_CLEAN;
-    if (!SCAN_ALL && ctx->num_viruses != 0)
-        if (ctx->options & CL_SCAN_HEURISTIC_PRECEDENCE)
+    if (!SCAN_ALLMATCHES && ctx->num_viruses != 0)
+        if (SCAN_HEURISTIC_PRECEDENCE)
             return CL_CLEAN;
-    if (ctx->limit_exceeded == 0 || SCAN_ALL) { 
+    if (ctx->limit_exceeded == 0 || SCAN_ALLMATCHES) {
         ctx->num_viruses++;
         *ctx->virname = virname;
         cli_virus_found_cb(ctx);
     }
 #if HAVE_JSON
-    if (SCAN_PROPERTIES && ctx->wrkproperty) {
+    if (SCAN_COLLECT_METADATA && ctx->wrkproperty) {
         json_object *arrobj, *virobj;
         if (!json_object_object_get_ex(ctx->wrkproperty, "Viruses", &arrobj)) {
             arrobj = json_object_new_array();
@@ -1214,7 +1248,7 @@ int
 cli_rmdirs(const char *name)
 {
 	int rc;
-	STATBUF statb;	
+	STATBUF statb;
 	DIR *dd;
 	struct dirent *dent;
 #if defined(HAVE_READDIR_R_3) || defined(HAVE_READDIR_R_2)
@@ -1277,7 +1311,7 @@ cli_rmdirs(const char *name)
 	return -1;
     }
 
-    return rc;	
+    return rc;
 }
 #else
 int cli_rmdirs(const char *dirname)
@@ -1356,7 +1390,7 @@ int cli_rmdirs(const char *dirname)
 	    rewinddir(dd);
 	}
 
-    } else { 
+    } else {
 	return -1;
     }
 
@@ -1386,7 +1420,7 @@ static unsigned long nearest_power(unsigned long num)
 bitset_t *cli_bitset_init(void)
 {
 	bitset_t *bs;
-	
+
 	bs = cli_malloc(sizeof(bitset_t));
 	if (!bs) {
         cli_errmsg("cli_bitset_init: Unable to allocate memory for bs %llu\n", (long long unsigned)sizeof(bitset_t));
@@ -1417,7 +1451,7 @@ static bitset_t *bitset_realloc(bitset_t *bs, unsigned long min_size)
 {
 	unsigned long new_length;
 	unsigned char *new_bitset;
-	
+
 	new_length = nearest_power(min_size);
 	new_bitset = (unsigned char *) cli_realloc(bs->bitset, new_length);
 	if (!new_bitset) {
@@ -1432,7 +1466,7 @@ static bitset_t *bitset_realloc(bitset_t *bs, unsigned long min_size)
 int cli_bitset_set(bitset_t *bs, unsigned long bit_offset)
 {
 	unsigned long char_offset;
-	
+
 	char_offset = bit_offset / BITS_PER_CHAR;
 	bit_offset = bit_offset % BITS_PER_CHAR;
 
@@ -1449,11 +1483,11 @@ int cli_bitset_set(bitset_t *bs, unsigned long bit_offset)
 int cli_bitset_test(bitset_t *bs, unsigned long bit_offset)
 {
 	unsigned long char_offset;
-	
+
 	char_offset = bit_offset / BITS_PER_CHAR;
 	bit_offset = bit_offset % BITS_PER_CHAR;
 
-	if (char_offset >= bs->length) {	
+	if (char_offset >= bs->length) {
 		return FALSE;
 	}
 	return (bs->bitset[char_offset] & ((unsigned char)1 << bit_offset));
