@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015-2018 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -71,7 +71,7 @@
  * in re-enabling affected modules.
  */
 
-#define CL_FLEVEL 102
+#define CL_FLEVEL 106
 #define CL_FLEVEL_DCONF	CL_FLEVEL
 #define CL_FLEVEL_SIGTOOL CL_FLEVEL
 
@@ -286,6 +286,7 @@ struct cl_engine {
     uint64_t engine_options;
 
     /* Limits */
+    uint32_t maxscantime;  /* Time limit (in milliseconds) */
     uint64_t maxscansize;  /* during the scanning of archives this size
 				     * will never be exceeded
 				     */
@@ -405,9 +406,6 @@ struct cl_engine {
     uint32_t maxiconspe; /* max number of icons to scan for PE */
     uint32_t maxrechwp3; /* max recursive calls for HWP3 parsing */
 
-    /* millisecond time limit for preclassification scanning */
-    uint32_t time_limit;
-
     /* PCRE matching limitations */
     uint64_t pcre_match_limit;
     uint64_t pcre_recmatch_limit;
@@ -429,6 +427,7 @@ struct cl_settings {
     uint32_t ac_maxdepth;
     char *tmpdir;
     uint32_t keeptmp;
+    uint32_t maxscantime;
     uint64_t maxscansize;
     uint64_t maxfilesize;
     uint32_t maxreclevel;
@@ -496,6 +495,7 @@ extern int have_rar;
 #define SCAN_COLLECT_METADATA                   (ctx->options->general & CL_SCAN_GENERAL_COLLECT_METADATA)
 #define SCAN_HEURISTICS                         (ctx->options->general & CL_SCAN_GENERAL_HEURISTICS)
 #define SCAN_HEURISTIC_PRECEDENCE               (ctx->options->general & CL_SCAN_GENERAL_HEURISTIC_PRECEDENCE)
+#define SCAN_UNPRIVILEGED                       (ctx->options->general & CL_SCAN_GENERAL_UNPRIVILEGED)
 
 #define SCAN_PARSE_ARCHIVE                      (ctx->options->parse & CL_SCAN_PARSE_ARCHIVE)
 #define SCAN_PARSE_ELF                          (ctx->options->parse & CL_SCAN_PARSE_ELF)
@@ -538,7 +538,7 @@ extern int have_rar;
 		     (((v) & 0x00ff000000000000ULL) >> 40) | \
 		     (((v) & 0xff00000000000000ULL) >> 56))
 
-#ifndef HAVE_ATTRIB_PACKED 
+#ifndef HAVE_ATTRIB_PACKED
 #define __attribute__(x)
 #endif
 #ifdef HAVE_PRAGMA_PACK
@@ -750,19 +750,28 @@ int cli_writen(int fd, const void *buff, unsigned int count);
 const char *cli_gettmpdir(void);
 
 /**
- * @brief Generate tempfile filename (no path) with a random MD5 hash.
- * 
+ * @brief Sanitize a relative path, so it cannot have a negative depth.
+ *
  * Caller is responsible for freeing the filename.
- * 
+ *
+ * @return char* filename or NULL.
+ */
+char *cli_sanitize_filepath(const char *filepath, size_t filepath_len);
+
+/**
+ * @brief Generate tempfile filename (no path) with a random MD5 hash.
+ *
+ * Caller is responsible for freeing the filename.
+ *
  * @return char* filename or NULL.
  */
 char *cli_genfname(const char *prefix);
 
 /**
  * @brief Generate a full tempfile filepath with a random MD5 hash and prefix the name, if provided.
- * 
+ *
  * Caller is responsible for freeing the filename.
- * 
+ *
  * @param dir 	 Alternative temp directory. (optional)
  * @return char* filename or NULL.
  */
@@ -770,9 +779,9 @@ char* cli_gentemp_with_prefix(const char* dir, const char* prefix);
 
 /**
  * @brief Generate a full tempfile filepath with a random MD5 hash.
- * 
+ *
  * Caller is responsible for freeing the filename.
- * 
+ *
  * @param dir 	 Alternative temp directory. (optional)
  * @return char* filename or NULL.
  */
@@ -784,38 +793,37 @@ char *cli_gentemp(const char *dir);
  * @param dir        Alternative temp directory (optional).
  * @param[out] name  Allocated filepath, must be freed by caller.
  * @param[out] fd    File descriptor of open temp file.
- * @return cl_error_t CL_SUCCESS, CL_ECREAT, or CL_EMEM. 
+ * @return cl_error_t CL_SUCCESS, CL_ECREAT, or CL_EMEM.
  */
 cl_error_t cli_gentempfd(const char *dir, char **name, int *fd);
 
 /**
  * @brief Create a temp filename, create the file, open it, and pass back the filepath and open file descriptor.
- * 
+ *
  * @param dir        Alternative temp directory (optional).
  * @param prefix  	 (Optional) Prefix for new file tempfile.
  * @param[out] name  Allocated filepath, must be freed by caller.
  * @param[out] fd    File descriptor of open temp file.
- * @return cl_error_t CL_SUCCESS, CL_ECREAT, or CL_EMEM. 
+ * @return cl_error_t CL_SUCCESS, CL_ECREAT, or CL_EMEM.
  */
 cl_error_t cli_gentempfd_with_prefix(const char* dir, char* prefix, char** name, int* fd);
 
 unsigned int cli_rndnum(unsigned int max);
 int cli_filecopy(const char *src, const char *dest);
-int cli_mapscan(fmap_t *map, off_t offset, size_t size, cli_ctx *ctx, cli_file_t type);
 bitset_t *cli_bitset_init(void);
 void cli_bitset_free(bitset_t *bs);
 int cli_bitset_set(bitset_t *bs, unsigned long bit_offset);
 int cli_bitset_test(bitset_t *bs, unsigned long bit_offset);
 const char* cli_ctime(const time_t *timep, char *buf, const size_t bufsize);
 void cli_check_blockmax(cli_ctx *, int);
-int cli_checklimits(const char *, cli_ctx *, unsigned long, unsigned long, unsigned long);
-int cli_updatelimits(cli_ctx *, unsigned long);
+cl_error_t cli_checklimits(const char *, cli_ctx *, unsigned long, unsigned long, unsigned long);
+cl_error_t cli_updatelimits(cli_ctx *, unsigned long);
 unsigned long cli_getsizelimit(cli_ctx *, unsigned long);
 int cli_matchregex(const char *str, const char *regex);
 void cli_qsort(void *a, size_t n, size_t es, int (*cmp)(const void *, const void *));
 void cli_qsort_r(void *a, size_t n, size_t es, int (*cmp)(const void*, const void *, const void *), void *arg);
-int cli_checktimelimit(cli_ctx *ctx);
-int cli_append_possibly_unwanted(cli_ctx * ctx, const char * virname);
+cl_error_t cli_checktimelimit(cli_ctx *ctx);
+cl_error_t cli_append_possibly_unwanted(cli_ctx *ctx, const char *virname);
 
 /* symlink behaviour */
 #define CLI_FTW_FOLLOW_FILE_SYMLINK 0x01
@@ -844,11 +852,11 @@ struct cli_ftw_cbdata {
     void *data;
 };
 
-/* 
+/*
  * return CL_BREAK to break out without an error, CL_SUCCESS to continue,
  * or any CL_E* to break out due to error.
  * The callback is responsible for freeing filename when it is done using it.
- * Note that callback decides if directory traversal should continue 
+ * Note that callback decides if directory traversal should continue
  * after an error, we call the callback with reason == error,
  * and if it returns CL_BREAK we break.
  */
@@ -861,7 +869,7 @@ typedef int (*cli_ftw_cb)(STATBUF *stat_buf, char *filename, const char *path, e
 typedef int (*cli_ftw_pathchk)(const char *path, struct cli_ftw_cbdata *data);
 
 /*
- * returns 
+ * returns
  *  CL_SUCCESS if it traversed all files and subdirs
  *  CL_BREAK if traversal has stopped at some point
  *  CL_E* if error encountered during traversal and we had to break out
@@ -880,10 +888,10 @@ const char *cli_strerror(int errnum, char* buf, size_t len);
 
 /**
  * @brief   Attempt to get a filename from an open file descriptor.
- * 
+ *
  * Caller is responsible for free'ing the filename.
  * Should work on Linux, macOS, Windows.
- * 
+ *
  * @param desc           File descriptor
  * @param[out] filepath  Will be set to file path if found, or NULL.
  * @return cl_error_t    CL_SUCCESS if found, else an error code.
